from setuptools import setup

setup(name='tcp-messages',description='create json services easily',author='german espinosa',author_email='germanespinosa@gmail.com',packages=['tcp_messages'],install_requires=['json-cpp>=1.0.78'],license='MIT',version='1.0.42',zip_safe=False)
